#include <p18f4550.h>
#include <stdio.h>
#include <stdlib.h> 
#include <string.h>
#include <adc.h>
//#include <delays.h>
//#include <timers.h>
#include <usart.h>

#pragma config PLLDIV = 1, CPUDIV = OSC1_PLL2, USBDIV = 2, FOSC =XT_XT, FCMEN = OFF, IESO = OFF, PWRT = ON             
#pragma config BOR = SOFT, BORV = 2, VREGEN = ON, WDT = OFF, WDTPS = 128, MCLRE = ON, LPT1OSC = OFF, PBADEN = OFF               
#pragma config CCP2MX = ON, STVREN = ON, LVP = OFF, ICPRT = OFF, XINST = OFF, DEBUG = OFF          
#pragma config CP0 = OFF, CP1 = OFF, CP2 = OFF,CP3 = OFF,CPB = OFF,CPD = OFF
#pragma config WRT0 = OFF, WRT1 = OFF, WRT2 = OFF, WRT3 = OFF, WRTB = OFF, WRTC = OFF, WRTD = OFF 
#pragma config EBTR0 = OFF, EBTR1 = OFF, EBTR2 = OFF, EBTR3 = OFF, EBTRB = OFF 




/*******************************************************************************/
/**************************  PROTOTYPE DEFINITIONS *****************************/
/*******************************************************************************/
void ISR_LOW_PRIORITY (void);
void ISR_HIGH_PRIORITY (void);
void Msg_Decoder (void);
void Get_Length (void);
void Get_Restriction (void);
void Get_Maintenance (void);
void Compute (void);
int Char_to_int (char *String, int Position);
void Line_Feed(void);


/*******************************************************************************/
/**************************  	INT VECTORS		   *****************************/
/*******************************************************************************/
#pragma code InterrupcionHigh = 0x0008 
void VectorHighInterrupcion(void){ 
_asm goto ISR_HIGH_PRIORITY _endasm 
}
#pragma code


#pragma code InterrupcionLow = 0x00018 
void VectorLowInterrupcion(void){ 
_asm goto ISR_LOW_PRIORITY _endasm 
} 
#pragma code

/*******************************************************************************/
/**************************  VARIABLE DEFINITIONS  *****************************/
/*******************************************************************************/
volatile unsigned int RX_control =0;
volatile unsigned int RX_index=0,index=0,fin=0;
volatile unsigned int Config_Status=0;
volatile char Recived_MSG[11];
unsigned char MSG1[] ="CONFIGURE";
unsigned int Length=0;
unsigned int Restriction=0;
unsigned int Maintenance=0;
signed float Flecha=0;
signed double Coef_F_L=0;
unsigned int MaxAL=0;
unsigned int Al_Maintenance=0;
signed float Strain=0;
signed float Strain_Maintenance=0;
signed float Gauge_factor=2.1;
signed float VM1=0;
signed float VM1_Maintenance=0;
signed float VM1_Neg=0;
signed float VM1_Maintenance_Neg=0;
signed float V1=6;
unsigned int AO_Gain=45;
volatile float Channel_0=0;
volatile float Channel_1=0;
volatile float Channel_0_M=0;
volatile float Channel_1_M=0;
volatile float Offset_CH_0=0;
volatile float Offset_CH_1=0;

/*******************************************************************************/
/**************************    ISR  ROUTINES       *****************************/
/*******************************************************************************/

#pragma interrupt ISR_LOW_PRIORITY
void ISR_LOW_PRIORITY (void){ 
	if (PIR1bits.RCIF==1){};
}


#pragma interrupt ISR_HIGH_PRIORITY
void ISR_HIGH_PRIORITY (void){ 
	unsigned int a;
	if (PIR1bits.RCIF==1){
		PIR1bits.RCIF=0;
		INTCONbits.PEIE=0; 	
		INTCONbits.GIE=0;
			while (!DataRdyUSART());
			Recived_MSG[RX_index]=getcUSART();
			if (Recived_MSG[RX_index]==0x0A){
					Recived_MSG[RX_index-1]=0x00;
					RX_index=0;
					fin=1;					
			}
	if (!fin && Config_Status==0){RX_index++;}	
	if (!fin && Config_Status==1){RX_index++;}	
	if (!fin && Config_Status==2){RX_index++;}
	if (!fin && Config_Status==3){RX_index++;}
	if (fin && Config_Status==0) {Msg_Decoder ();}
	if (fin && Config_Status==1) {Get_Length (); }
	if (fin && Config_Status==2) {Get_Restriction (); }
	if (fin && Config_Status==3) {Get_Maintenance (); }
	if (RX_index > 11){
		Line_Feed();
		printf (" Buffer overflow, System RESET" );
		Line_Feed();
		Delay100TCYx (0);
		Reset();	}		
		}	

		INTCONbits.PEIE=1; 	
		INTCONbits.GIE=1;
	}

/*******************************************************************************/
/**************************			  MAIN	   	   *****************************/
/*******************************************************************************/
void main(void){
OpenUSART(USART_TX_INT_OFF & USART_RX_INT_ON & USART_ASYNCH_MODE & USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH,25);
OpenADC(ADC_FOSC_RC & ADC_12_TAD & ADC_RIGHT_JUST, ADC_REF_VREFPLUS_VSS & ADC_INT_OFF, ADC_3ANA);
RCONbits.IPEN=1; 		
INTCONbits.PEIE=1; 			
INTCONbits.GIE=1;

    while(BusyUSART());
		printf (" Structural integrity V 1.0 \n");
		Line_Feed();
		printf (0x0D);
		TRISDbits.TRISD0=0;
		TRISDbits.TRISD1=0;
		TRISDbits.TRISD2=0;
		PORTDbits.RD0=0;
		PORTDbits.RD1=0;
		PORTDbits.RD2=0;
    while(1){
		if (Config_Status<5){
			PORTDbits.RD0=1;
		}
		if (Config_Status==5){
			PORTDbits.RD0=0;
		}
		
		if (Config_Status==5){
			SetChanADC (ADC_CH0);
			ConvertADC();
		while (BusyADC()==1){}
			Channel_0_M= ReadADC();
			Channel_0=Channel_0_M*0.005859;
			Channel_0=Channel_0-Offset_CH_0;
			if (Channel_0 > VM1_Maintenance | Channel_0 < VM1_Maintenance_Neg){
				PORTDbits.RD1=1;
				Line_Feed();
				printf ("DEFORMATION OF SERVICE REACHED");
				Line_Feed();
				}else {PORTDbits.RD1=0;}
			if (Channel_0 > VM1 | Channel_0 < VM1_Neg){
				PORTDbits.RD2=1;
				Line_Feed();
				printf ("REACHED SAFETY DEFORMATION");
				Line_Feed();
				}else {PORTDbits.RD2=0;}
		}
		if (Config_Status==5){
			SetChanADC (ADC_CH1);
			ConvertADC();
		while (BusyADC()==1){}
			Channel_1_M= ReadADC();
			Channel_1=Channel_1_M*0.005859;
			Channel_1=Channel_1-Offset_CH_1;
			if (Channel_1 > VM1_Maintenance | Channel_1 < VM1_Maintenance_Neg){
				PORTDbits.RD1=1;
				Line_Feed();
				printf ("DEFORMATION OF SERVICE REACHED");
				Line_Feed();
				}else {PORTDbits.RD1=0;}
			if (Channel_1 > VM1 | Channel_1 < VM1_Neg){
				PORTDbits.RD2=1;
				Line_Feed();
				printf ("REACHED SAFETY DEFORMATION");
				Line_Feed();
				}else {PORTDbits.RD2=0;}
		}
   	}                    
}

void Msg_Decoder (void){
unsigned int i=0;
i = memcmp( MSG1, Recived_MSG, 9 );
	if (!i==0){
		printf ("Sintax Error \n");
		Line_Feed();
		fin=0;}
	if (i==0){
		fin=0;
		Config_Status=1;
		printf ("Please input the beam length in mm. \n");
		Line_Feed();}
}


void Get_Length (void){
unsigned int valor=0;
valor=Char_to_int ( Recived_MSG, strlen (Recived_MSG));
	if (valor==0){
		printf ("Please input a decimal value, Maximum 99999 mm.\n");
		Line_Feed();
		fin=0;}
	if (valor>0){
		Length=valor;
		Config_Status=2;
		printf ("Please choose the beam deflection restriction: \n");
		Line_Feed();
		printf ("1. 1/250	2. 1/300	3. 1/400	4. 1/500 \n");
		Line_Feed();
		fin=0;}
}

void Get_Restriction (void){
unsigned int valor=0;
valor=Char_to_int ( Recived_MSG, strlen (Recived_MSG));
	if (valor==0){
		printf ("Please select a decimal value 1,2,3,4 \n");
		Line_Feed();
		fin=0;}
	if (valor<0 | valor>4) {
		printf ("Please select a value between  1 to 4 \n");
		Line_Feed();
		fin=0;}
	if (valor>0 & valor<5) {
	if (valor==1){ Restriction=250;}
	if (valor==2){ Restriction=300;}
	if (valor==3){ Restriction=400;}
	if (valor==4){ Restriction=500;}
	Config_Status=3;
	printf ("Please select the alarm maintenance value (25 to 75) :\n");
	Line_Feed();
	fin=0;
	}
}
void Get_Maintenance (void){
unsigned int valor=0;
valor=Char_to_int ( Recived_MSG, strlen (Recived_MSG));
	if (valor==0){
		printf ("Please input a decimal value between 25 to 75 \n");
		Line_Feed();
		fin=0;}
	if (valor<25 | valor>75){
		Line_Feed();
		printf ("Please input maintenance value between 25 to 75 :  \n");
		fin=0;}
	if (valor>24 & valor<76) {
		Maintenance=valor;
		Config_Status=4;
		Line_Feed();
		printf ("Please wait, parameters are being computed \n");
		Compute ();
		fin=0;}
}
void Compute (void){
signed double aux=0;
aux=Length;
Flecha=aux/Restriction;
Flecha=Flecha;
Coef_F_L=1000/Restriction;
Coef_F_L=Coef_F_L/1000;
aux=Coef_F_L/100;
aux=Coef_F_L+aux;
MaxAL=Flecha/aux;
aux=Length-MaxAL;
aux=aux/100;
Al_Maintenance=aux*Maintenance;
Al_Maintenance=Length-Al_Maintenance;
Strain=Length-MaxAL;
Strain=Strain/Length;
Strain_Maintenance=Length-Al_Maintenance;
Strain_Maintenance=Strain_Maintenance/Length;
Gauge_factor=Gauge_factor*(-1);
VM1=Gauge_factor*Strain;
VM1=VM1/2;
VM1=VM1*V1;
VM1=VM1*40;
VM1=VM1-3;
VM1=VM1*(-1);

if (VM1>6){
	Line_Feed();
	printf ( "Device configuration no available, SYSTEM RESET \n");
	Reset();
	Line_Feed();
	}

VM1_Maintenance=Gauge_factor*Strain_Maintenance;
VM1_Maintenance=VM1_Maintenance/2;
VM1_Maintenance=VM1_Maintenance*V1;
VM1_Maintenance=VM1_Maintenance*40;
VM1_Maintenance=VM1_Maintenance-3;
VM1_Maintenance=VM1_Maintenance*(-1);

Gauge_factor=Gauge_factor*(-1);
VM1_Neg=Gauge_factor*Strain;
VM1_Neg=VM1_Neg/2;
VM1_Neg=VM1_Neg*V1;
VM1_Neg=VM1_Neg*40;
VM1_Neg=VM1_Neg-3;
VM1_Neg=VM1_Neg*(-1);

VM1_Maintenance_Neg=Gauge_factor*Strain_Maintenance;
VM1_Maintenance_Neg=VM1_Maintenance_Neg/2;
VM1_Maintenance_Neg=VM1_Maintenance_Neg*V1;
VM1_Maintenance_Neg=VM1_Maintenance_Neg*40;
VM1_Maintenance_Neg=VM1_Maintenance_Neg-3;
VM1_Maintenance_Neg=VM1_Maintenance_Neg*(-1);

Line_Feed();
printf ( "Device configuration OK \n");
Line_Feed();
printf ( "Please wait, Channel 1 Offset measuring... \n" );
Line_Feed();
SetChanADC (ADC_CH0);
Delay10TCYx(1);
ConvertADC();
while (BusyADC()==1){}
Offset_CH_0 = ReadADC();
//Offset_CH_0=Offset_CH_0-25;
Offset_CH_0=Offset_CH_0*0.005859;
Offset_CH_0=Offset_CH_0-3;
printf ( "Please wait, Channel 2 Offset measuring... \n" );
Line_Feed();
SetChanADC (ADC_CH1);
Delay10TCYx(1);
ConvertADC();
while (BusyADC()==1){}
Offset_CH_1 = ReadADC();
//Offset_CH_1=Offset_CH_1-25;
Offset_CH_1=Offset_CH_1*0.005859;
Offset_CH_1=Offset_CH_1-3;

printf ( "The system is READY to work \n" );
Line_Feed();
Config_Status=5;
PORTDbits.RD0=0;
}

int Char_to_int (char *String, int Position){
unsigned int  Dec_Millar=0;
unsigned int  Millar=0;
unsigned int Centena=0;
unsigned int Decena=0;
unsigned int Unidad=0;
unsigned int index=0;
unsigned int index2=0;
unsigned char numero[5];

	for (index=(5-Position); index<5; index++){
		numero[index]=Recived_MSG[index2];
		index2++;
		}
	for (index=0; index<(5-Position); index++){	
		numero[index]=48;
		}
	
	Dec_Millar=(numero[0]-48);
	if (Dec_Millar > 9 | Dec_Millar<0){ return 0;}
	Dec_Millar=Dec_Millar*10000;
	Millar=(numero[1]-48);
	if (Millar > 9 | Millar<0){ return 0;}
	Millar=	Millar*1000;
	Centena=(numero[2]-48);
	if (Centena > 9 | Centena<0){ return 0;}
	Centena=Centena*100;
	Decena=(numero[3]-48);
	if (Decena > 9 | Decena<0){ return 0;}
	Decena=Decena*10;
	Unidad=(numero[4]-48);
	if (Unidad > 9 | Unidad<0){ return 0;}
	return (Dec_Millar+Millar+Centena+Decena+Unidad);
}

void Line_Feed(void){
putcUSART(0x0D);
}

